@@ -55,6 +55,7 @@ import termios
 import tty
 import signal
 import re
+import time
 import collections
 
 PY2 = sys.version_info < (3, 0)
@@ -344,6 +345,7 @@ class Flootty(object):
         self.term_id = None
         self.orig_stdin_atts = None
         self.orig_stdout_atts = None
+        self.last_stdin = 0
 
     def add_fd(self, fileno, **kwargs):
         try:
@@ -696,7 +698,7 @@ class Flootty(object):
         if self.child_pid == pty.CHILD:
             os.execlp(shell, shell, '--login')
 
-        self.orig_stdin_atts = tty.tcgetattr(sys.stdin)
+        self.orig_stdin_atts = tty.tcgetattr(sys.stdin.fileno())
         tty.setraw(pty.STDIN_FILENO)
         self.original_wincher = signal.signal(signal.SIGWINCH, self._signal_winch)
         self._set_pty_size()
@@ -710,22 +712,30 @@ class Flootty(object):
             '''
             Called when there is data to be sent from the child process back to the user.
             '''
-            data = self.extra_data + os.read(fd, FD_READ_BYTES)
+
+            try:
+                data = self.extra_data + os.read(fd, FD_READ_BYTES)
+            except:
+                data = None
+
+            if not data:
+                return die("Time to go!")
+
             self.extra_data = b''
-            if data:
-                while True:
-                    try:
-                        data.decode('utf-8')
-                    except UnicodeDecodeError:
-                        self.extra_data = data[-1:] + self.extra_data
-                        data = data[:-1]
-                    else:
-                        break
-                    if len(self.extra_data) > 100:
-                        die('not a valid utf-8 string: %s' % self.extra_data)
-                if data:
-                    self.transport("term_stdout", {'data': data.decode('utf-8'), 'id': self.term_id})
-                    write(pty.STDOUT_FILENO, data)
+
+            while True:
+                try:
+                    data.decode('utf-8')
+                except UnicodeDecodeError:
+                    self.extra_data = data[-1:] + self.extra_data
+                    data = data[:-1]
+                else:
+                    break
+                if len(self.extra_data) > 100:
+                    die('not a valid utf-8 string: %s' % self.extra_data)
+
+            self.transport("term_stdout", {'data': data.decode('utf-8'), 'id': self.term_id})
+            write(pty.STDOUT_FILENO, data)
 
         self.add_fd(self.master_fd, reader=stdout_write, errer=slave_death, name='create_term_stdout_write')
 
@@ -733,7 +743,12 @@ class Flootty(object):
             data = os.read(fd, FD_READ_BYTES)
             if data:
                 write(self.master_fd, data)
-                self.transport("term_stdin", {'data': data.decode('utf-8'), 'id': self.term_id})
+
+                now = time.time()
+                # Only send stdin event if it's been > 2 seconds. This prevents people from figuring out password lengths
+                if now - self.last_stdin > 2:
+                    self.transport("term_stdin", {'data': ' ', 'id': self.term_id})
+                    self.last_stdin = now
 
         self.add_fd(pty.STDIN_FILENO, reader=stdin_write, name='create_term_stdin_write')
 
@@ -747,10 +762,11 @@ class Flootty(object):
         # TODO: other shells probably use weird color escapes
         if 'zsh' in shell:
             color_start = "%{%F{green}%}"
-            color_reset = ""
+            color_reset = "%{%f%}"
 
-        set_prompt_command = 'PS1="%s%s::%s::%s%s $PS1"\n' % (color_start, self.owner, self.room, self.term_name, color_reset)
-        net_stdin_write(set_prompt_command)
+        # Set prompt
+        cmd = 'PS1="%s%s::%s::%s%s $PS1"\n' % (color_start, self.owner, self.room, self.term_name, color_reset)
+        net_stdin_write(cmd)
 
     def _signal_winch(self, signum, frame):
         '''
